<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Controller\Callback;

use ConnectPay\Payment\Gateway\Config\Config;
use Exception;
use Magento\Framework\App\Action\HttpPostActionInterface;
use Magento\Framework\App\CsrfAwareActionInterface;
use Magento\Framework\App\Request\InvalidRequestException;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\Exception\NotFoundException;
use Magento\Framework\Serialize\Serializer\Json;
use Magento\Framework\Webapi\Request;
use Magento\Payment\Gateway\Command\CommandException;
use Magento\Payment\Gateway\Command\CommandPoolInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectFactory;
use Magento\Sales\Api\OrderPaymentRepositoryInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Psr\Log\LoggerInterface as Logger;

class Index implements HttpPostActionInterface, CsrfAwareActionInterface
{
    /**
     * @var CommandPoolInterface
     */
    private $commandPool;

    /**
     * @var Request
     */
    private $request;

    /**
     * @var Json
     */
    private $serializer;

    /**
     * @var JsonFactory
     */
    private $resultJsonFactory;

    /**
     * @var Logger
     */
    private $logger;

    /**
     * @var OrderPaymentRepositoryInterface
     */
    private $paymentRepository;

    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;

    /**
     * @var PaymentDataObjectFactory
     */
    private $paymentDataObjectFactory;

    public function __construct(
        CommandPoolInterface            $commandPool,
        Request                         $request,
        Json                            $serializer,
        JsonFactory                     $resultJsonFactory,
        Logger                          $logger,
        OrderPaymentRepositoryInterface $paymentRepository,
        OrderRepositoryInterface        $orderRepository,
        PaymentDataObjectFactory        $paymentDataObjectFactory
    )
    {
        $this->commandPool = $commandPool;
        $this->request = $request;
        $this->serializer = $serializer;
        $this->resultJsonFactory = $resultJsonFactory;
        $this->logger = $logger;
        $this->paymentRepository = $paymentRepository;
        $this->orderRepository = $orderRepository;
        $this->paymentDataObjectFactory = $paymentDataObjectFactory;
    }

    /**
     * @return ResultInterface
     * @throws CommandException
     * @throws NotFoundException
     */
    public function execute(): ResultInterface
    {
        try {
            $data = $this->serializer->unserialize($this->request->getContent());
            if (array_key_exists(Config::KEY_IDENTIFIERS, $data) && is_array($data[Config::KEY_IDENTIFIERS])) {
                $identfiersArray = $data[Config::KEY_IDENTIFIERS];
                if (array_key_exists(Config::KEY_MERCHANT_REFERENCE, $identfiersArray) && isset($identfiersArray[Config::KEY_MERCHANT_REFERENCE])) {
                    $cpMerchantReference = (int)$identfiersArray[Config::KEY_MERCHANT_REFERENCE];
                    $payment = $this->paymentRepository->get($cpMerchantReference);
                    $orderId = $payment->getParentId();
                    $order = $this->orderRepository->get($orderId);
                    $data = [
                        'order' => $order,
                        'payment' => $this->paymentDataObjectFactory->create($order->getPayment())
                    ];
                }
            }
        } catch (Exception $exception) {
            $this->logger->critical($exception);
            throw new Exception(__('Response does not exist.'));
        }
        $this->commandPool->get(Config::CALLBACK_COMMAND)->execute($data);
        return $this->resultJsonFactory->create([]);
    }

    public function validateForCsrf(RequestInterface $request): ?bool
    {
        return true;
    }

    public function createCsrfValidationException(RequestInterface $request): ?InvalidRequestException
    {
        return null;
    }
}
