<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Cron;

use ConnectPay\Payment\Gateway\Config\Config;
use Magento\Framework\Exception\NotFoundException;
use Magento\Payment\Gateway\Command\CommandException;
use Magento\Payment\Gateway\Command\CommandPoolInterface;
use Magento\Payment\Gateway\Data\PaymentDataObjectFactory;
use Magento\Sales\Model\ResourceModel\Order\CollectionFactory as OrderCollectionFactory;

class GetPaymentDetails
{
    /**
     * @var OrderCollectionFactory
     */
    private $orderCollectionFactory;

    /**
     * @var PaymentDataObjectFactory
     */
    private $paymentDataObjectFactory;

    /**
     * @var CommandPoolInterface
     */
    private $commandPool;

    private int $page;

    /**
     * @param OrderCollectionFactory $orderCollectionFactory
     * @param PaymentDataObjectFactory $paymentDataObjectFactory
     * @param CommandPoolInterface $commandPool
     */
    public function __construct(
        OrderCollectionFactory   $orderCollectionFactory,
        PaymentDataObjectFactory $paymentDataObjectFactory,
        CommandPoolInterface     $commandPool
    )
    {
        $this->orderCollectionFactory = $orderCollectionFactory;
        $this->paymentDataObjectFactory = $paymentDataObjectFactory;
        $this->commandPool = $commandPool;
        $this->page = 1;
    }

    /**
     * @return void
     * @throws NotFoundException
     * @throws CommandException
     */
    public function execute()
    {
        $limit = 100;
        do {
            $orderCollection = $this->orderCollectionFactory->create()->addFieldToFilter('status', 'pending')->setPageSize($limit)->setCurPage($this->page);
            /* join with payment table */
            $joinQuery = $orderCollection->getSelect()
                ->join(["sop" => "sales_order_payment"],
                    'main_table.entity_id = sop.parent_id',
                    array('method')
                )
                ->where('sop.method = ?', Config::PAYMENT_CODE);
            $ordersArray = $orderCollection->load($joinQuery);
            $this->page++;
            foreach ($ordersArray as $order) {
                $data = [
                    'order' => $order,
                    'payment' => $this->paymentDataObjectFactory->create($order->getPayment())
                ];
                $this->commandPool->get(Config::CALLBACK_COMMAND)->execute($data);
            }
        } while (count($ordersArray) === $limit);
    }
}
