<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway\Request;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Gateway\SubjectReader;
use Magento\Framework\HTTP\PhpEnvironment\RemoteAddress;

class ConsumerDataBuilder extends AbstractDataBuilder
{
    /**
     * @var RemoteAddress
     */
    private $remoteAddress;

    /**
     * @param RemoteAddress $remoteAddress
     * @param SubjectReader $subjectReader
     */
    public function __construct(
        RemoteAddress $remoteAddress,
        SubjectReader $subjectReader
    )
    {
        $this->remoteAddress = $remoteAddress;
        parent::__construct($subjectReader);
    }

    /**
     * @param array $buildSubject
     * @return array[]
     */
    public function build(array $buildSubject): array
    {
        $paymentDO = $this->subjectReader->readPayment($buildSubject);
        $order = $paymentDO->getOrder();
        $customerId = $order->getCustomerId();
        $billing = $order->getBillingAddress();

        return [
            Config::KEY_CONSUMER => [
                Config::KEY_FIRST_NAME => $billing->getFirstname(),
                Config::KEY_LAST_NAME => $billing->getLastname(),
                Config::KEY_CONSUMER_ID => (string)is_null($customerId) ? $billing->getEmail() : $customerId,
                Config::KEY_IP_ADDRESS => $this->remoteAddress->getRemoteAddress(),
                Config::KEY_EMAIL => $billing->getEmail(),
            ]
        ];
    }
}
