<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Gateway;

use InvalidArgumentException;
use Magento\Payment\Gateway\Data\PaymentDataObjectInterface;
use ConnectPay\Payment\Gateway\Config\Config;

class SubjectReader
{
    /**
     * Read payment from subject
     *
     * @param array[][] $subject
     * @return PaymentDataObjectInterface
     * @throws InvalidArgumentException
     */
    public function readPayment(array $subject): PaymentDataObjectInterface
    {
        if (!isset($subject['payment'])
            || !$subject['payment'] instanceof PaymentDataObjectInterface
        ) {
            throw new InvalidArgumentException('Payment data object should be provided');
        }

        return $subject['payment'];
    }

    /**
     * Read payment from subject
     *
     * @param array $subject
     * @return array
     */
    public function readResponse(array $subject): array
    {
        if (!isset($subject['response']) || !is_array($subject['response'])) {
            throw new InvalidArgumentException('Response does not exist');
        }

        return $subject['response'];
    }

    /**
     * Read paymentId from subject
     *
     * @param array $subject
     * @return string
     */
    public function readPaymentId(array $subject): string
    {
        if (isset($subject['payment']) || is_array($subject['payment'])) {
            $paymentArray = $subject['payment'];
            if (!isset($paymentArray['paymentId'])) {
                throw new InvalidArgumentException('PaymentId does not exist');
            }
            return $paymentArray['paymentId'];
        }
        return '';
    }

    /**
     * @param array $subject
     * @return string
     */
    public function readIdentifiers(array $subject): string
    {
        if (!isset($subject[Config::KEY_IDENTIFIERS])) {
            throw new InvalidArgumentException('The "identifiers" field does not exists');
        }

        return $subject[Config::KEY_IDENTIFIERS];
    }

    public function readStatus(array $subject): array
    {
        if (!isset($subject['status']) || !is_array($subject['status'])) {
            throw new InvalidArgumentException('The "status" field does not exist');
        }

        return $subject['status'];
    }
}
