<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Model\Checkout;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Model\ImageProviderMapper;
use ConnectPay\Payment\Model\Request\Provider as ProviderData;
use Magento\Checkout\Model\ConfigProviderInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\View\Asset\Repository as AssetRepository;

class ConnectPayConfigProvider implements ConfigProviderInterface
{
    /**
     * @var AssetRepository
     */
    private $assetRepository;

    /**
     * @var Config
     */
    private $config;

    /**
     * @var ProviderData
     */
    private $provider;

    /**
     * @var ImageProviderMapper
     */
    private $mapper;

    public function __construct(
        AssetRepository     $assetRepository,
        Config              $config,
        ProviderData        $provider,
        ImageProviderMapper $mapper)
    {
        $this->assetRepository = $assetRepository;
        $this->config = $config;
        $this->provider = $provider;
        $this->mapper = $mapper;
    }

    /**
     * @return \array[][]
     * @throws LocalizedException
     */
    public function getConfig(): array
    {
        return [
            'payment' => [
                Config::CODE => [
                    'connectpayRedirectUrl' => Config::REDIRECT_URL,
                    'connectpayIcon' => $this->getIcon(),
                    'connectpaySelectedProviders' => $this->getProviders(),
                    'connectpayTermsPrivacy' => __('Payment Initiation is provided by ConnectPay UAB. By using our services, you agree to our <a href="https://connectpay.com/wp-content/uploads/2021/01/Terms-of-Use-for-PIS-and-AIS.pdf" target="_blank">Terms of Use</a> and <a href="https://connectpay.com/privacy-policy/" target="_blank">Privacy Policy</a>.')
                ]
            ]
        ];
    }

    /**
     * Get payment icon url
     *
     * @return string
     */
    private function getIcon(): string
    {
        $fileId = 'ConnectPay_Payment::images/connectpay-icon.png';
        $params = [];
        return $this->assetRepository->getUrlWithParams($fileId, $params);
    }

    /**
     * @return array
     * @throws LocalizedException
     */
    private function getProviders(): array
    {
        $providerCountry = $this->config->getProviderCountryCode();
        $providerArray = $this->provider->getProviders($providerCountry);

        return $this->mapper->getProviderData($providerArray, $this->config->getProviderId());
    }
}
