<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Model\Config\Source;

use ConnectPay\Payment\Gateway\Config\Config;
use ConnectPay\Payment\Model\ProviderMapper;
use ConnectPay\Payment\Model\Request\Provider as ProviderData;
use Magento\Framework\Data\OptionSourceInterface;
use Magento\Framework\Exception\LocalizedException;

class Provider implements OptionSourceInterface
{
    /**
     * @var Config
     */
    private $config;

    /**
     * @var ProviderData
     */
    private $provider;

    /**
     * @var ProviderMapper
     */
    private $mapper;

    /**
     * @param Config $config
     * @param ProviderData $provider
     */
    public function __construct(
        Config         $config,
        ProviderData   $provider,
        ProviderMapper $mapper
    )
    {
        $this->config = $config;
        $this->provider = $provider;
        $this->mapper = $mapper;
    }

    /**
     * @return array
     * @throws LocalizedException
     */
    public function toOptionArray(): array
    {
        $optionArray = [];

        $data = $this->getProviders();
        if (!empty($data)) {
            foreach ($data as $key => $value) {
                $optionArray[] = [
                    'value' => $value['id'], 'label' => $value['name']
                ];
            }
        }

        return $optionArray;
    }

    /**
     * @return array
     * @throws LocalizedException
     */
    protected function getProviders(): array
    {
        $providerCountry = $this->config->getProviderCountryCode();
        $providerData = $this->provider->getProviders($providerCountry);
        return $this->mapper->getProviderData($providerData);
    }
}
