<?php

declare(strict_types=1);

namespace ConnectPay\Payment\Model\Request;

use Magento\Framework\Serialize\Serializer\Json;

class Signature
{
    /**
     * @var Json
     */
    private $serializer;

    /**
     * @param Json $serializer
     */
    public function __construct(
        Json $serializer
    )
    {
        $this->serializer = $serializer;
    }

    /**
     * @param array $data
     * @param string $requestMethod
     * @param string $url
     * @param string $endpoint
     * @param  $pkeyid
     * @return string
     */
    public function getSignature(array $data, string $requestMethod, string $url, string $endpoint, $pkeyid): string
    {
        $signature = '';
        $reqMethod = strtolower($requestMethod);
        $reqUrl = strtolower(parse_url($url, PHP_URL_HOST));
        $reqEndpoint = strtolower($endpoint);
        $queryData = $this->getFromRequestData($data, 'query');
        $reqQuery = $this->formatQuery($queryData);
        if (!empty($reqQuery)) {
            $cleanUrl = $reqUrl . '/' . $reqEndpoint . '?' . $reqQuery;
        } else {
            $cleanUrl = $reqUrl . '/' . $reqEndpoint;
        }

        $bodyData = $this->getFromRequestData($data, 'body');
        $reqBody = $this->formatBody($bodyData);

        $dataToSign = $reqMethod . '|' . $cleanUrl . '|' . $reqBody;
        openssl_sign($dataToSign, $signature, $pkeyid, OPENSSL_ALGO_SHA256);
        openssl_free_key($pkeyid);

        return base64_encode($signature);
    }

    /**
     * @param $data
     * @param string $key
     * @return mixed|string
     */
    protected function getFromRequestData($data, string $key)
    {
        if (array_key_exists($key, $data)) {
            if (!empty($data[$key])) {
                return $data[$key];
            }
        }
        return '';
    }

    /**
     * @param $data
     * @return string
     */
    protected function formatQuery($data): string
    {
        $resultString = '';
        if (!empty($data)) {
            if (count($data) > 1) {
                foreach ($data as $key => $value) {
                    $resultString .= $key . '=' . $value . '&';
                }
                $resultString = substr($resultString, 0, -1);
            } else {
                foreach ($data as $key => $value) {
                    $resultString .= $key . '=' . $value;
                }
            }

            return strtolower($resultString);
        }
        return $resultString;
    }

    /**
     * @param string $data
     * @return string
     */
    protected function formatBody(string $data): string
    {
        $dataArr = $this->serializer->unserialize($data);
        if (empty($dataArr)) {
            return '';
        }
        return preg_replace('/\s+/', '', $data);
    }

}
